from django.core.management.base import BaseCommand
from wq.db.patterns.identify.models import Authority


class Command(BaseCommand):
    def handle(self, *args, **options):
        Authority.objects.get_or_create(
            name="This Site",
        )
        Authority.objects.get_or_create(
            name="Wikipedia",
            homepage="https://wikipedia.org",
            object_url="https://wikipedia.org/wiki/%s",
        )



import unittest
from .base import APITestCase
from rest_framework import status
import json
from tests.rest_app.models import SlugModel
from tests.gis_app.models import GeometryModel
from django.contrib.auth.models import User
from django.conf import settings


class RestPostTestCase(APITestCase):
    def setUp(self):
        self.user = User.objects.create(username="testuser", is_superuser=True)
        self.client.force_authenticate(self.user)

    @unittest.skipUnless(settings.WITH_GIS, "requires GIS")
    def test_rest_geometry_post_geojson(self):
        """
        Posting GeoJSON to a model with a geometry field should work.
        """
        form = {
            'name': "Geometry Test 1",
            'geometry': json.dumps({
                "type": "Point",
                "coordinates": [-90, 44]
            })
        }

        # Test for expected response
        response = self.client.post('/geometrymodels.json', form)
        self.assertEqual(
            response.status_code, status.HTTP_201_CREATED, response.data
        )

        # Double-check ORM model & geometry attribute
        obj = GeometryModel.objects.get(id=response.data['id'])
        geom = obj.geometry
        self.assertEqual(geom.srid, 4326)
        self.assertEqual(geom.x, -90)
        self.assertEqual(geom.y, 44)

    @unittest.skipUnless(settings.WITH_GIS, "requires GIS")
    def test_rest_geometry_post_wkt(self):
        """
        Posting WKT to a model with a geometry field should work.
        """
        form = {
            'name': "Geometry Test 2",
            'geometry': "POINT(%s %s)" % (-97, 50)
        }

        # Test for expected response
        response = self.client.post('/geometrymodels.json', form)
        self.assertEqual(
            response.status_code, status.HTTP_201_CREATED, response.data
        )

        # Double-check ORM model & geometry attribute
        obj = GeometryModel.objects.get(id=response.data['id'])
        geom = obj.geometry
        self.assertEqual(geom.srid, 4326)
        self.assertEqual(geom.x, -97)
        self.assertEqual(geom.y, 50)

    def test_rest_date_label_post(self):
        """
        Posting to a model with a date should return a label and an ISO date
        """
        form = {
            'name': "Test Date",
            'date': '2015-06-01 12:00:00Z',
        }
        response = self.client.post("/datemodels.json", form)
        self.assertEqual(
            response.status_code, status.HTTP_201_CREATED, response.data
        )
        self.assertIn('date_label', response.data)
        self.assertEqual(response.data['date_label'], "2015-06-01 07:00 AM")
        self.assertIn('date', response.data)
        self.assertEqual(response.data['date'], "2015-06-01T07:00:00-05:00")

    def test_rest_empty_date_post(self):
        """
        Allow posting an empty date if the field allows nulls
        """
        form = {
            'name': "Test Date",
            'date': '2015-06-01 12:00:00Z',
            'empty_date': '',
        }
        response = self.client.post("/datemodels.json", form)
        self.assertEqual(
            response.status_code, status.HTTP_201_CREATED, response.data
        )

    def test_rest_empty_string(self):
        def check_result(form, expected_output):
            response = self.client.post("/charfieldmodels.json", form)
            if not expected_output:
                self.assertEqual(
                    response.status_code,
                    status.HTTP_400_BAD_REQUEST,
                    response.data
                )
                return

            self.assertEqual(
                response.status_code, status.HTTP_201_CREATED, response.data
            )
            for field, value in expected_output.items():
                if value is None:
                    self.assertIsNone(response.data[field])
                else:
                    self.assertEqual(response.data[field], value)

        # No fields submitted
        check_result({}, False)

        # Required field submitted but left blank
        check_result({'required_field': ''}, False)

        # Required field submitted
        check_result({
            'required_field': 'test',
        }, {
            'required_field': 'test',
            'nullable_field': None,
            'blankable_field': '',
            'nullableblankable_field': None,
        })

        # All fields submitted but left blank
        check_result({
            'required_field': 'test',
            'nullable_field': '',
            'blankable_field': '',
            'nullableblankable_field': '',
        }, {
            'required_field': 'test',
            'nullable_field': None,
            'blankable_field': '',
            'nullableblankable_field': '',
        })

    def test_rest_custom_lookup_fk(self):
        SlugModel.objects.create(
            code='test1',
            name='Test #1',
        )
        response = self.client.post('/slugrefparents.json', {
            'ref_id': 'test1',
            'name': "Test FK",
        })
        self.assertTrue(status.is_success(response.status_code), response.data)
        rid = response.data.get('id')
        self.assertEqual(response.data, {
            'id': rid,
            'label': 'Test FK (Test #1)',
            'name': 'Test FK',
            'ref_id': 'test1',
            'ref_label': 'Test #1',
        })

        response = self.client.post('/slugrefparents.json', {
            'ref_id': 'test_invalid',
            'name': "Test FK",
        })
        self.assertTrue(
            status.is_client_error(response.status_code), response.data
        )
        self.assertEqual(response.data, {
            'ref_id': ['Object with code=test_invalid does not exist.']
        })

    def test_rest_list_exclude_post(self):
        # Create
        response = self.client.post('/expensivemodels.json', {
            "name": "Test",
            "expensive": "SOME_DATA",
        })
        item = response.data
        item_url = '/expensivemodels/{pk}.json'.format(pk=item['id'])
        self.assertEqual(item['name'], "Test")
        self.assertEqual(item['expensive'], "SOME_DATA")

        # List view
        response = self.client.get('/expensivemodels.json')
        item = response.data['list'][0]
        self.assertEqual(item['name'], "Test")
        self.assertNotIn('expensive', item)
        self.assertNotIn('more_expensive', item)

        # Detail view
        response = self.client.get(item_url)
        item = response.data
        self.assertEqual(item['name'], "Test")
        self.assertEqual(item['expensive'], "SOME_DATA")

        # Update
        response = self.client.put(item_url, {
            "name": "Test 2",
            "expensive": "SOME_OTHER_DATA",
        })
        item = response.data
        self.assertEqual(item['name'], "Test 2")
        self.assertEqual(item['expensive'], "SOME_OTHER_DATA")



from wq.db import rest
from .models import GeometryModel, PointModel
from django.conf import settings

if settings.WITH_GIS:
    rest.router.register_model(
        GeometryModel,
        fields="__all__",
    )
    rest.router.register_model(
        PointModel,
        fields="__all__",
    )



from wq.db import rest
from .models import (
    RootModel, UserManagedModel, Parent, Child, ItemType, Item,
    FileModel, ImageModel,
    SlugModel, SlugRefParent, SlugRefChild,
    DateModel, ChoiceModel, TranslatedModel,
    CharFieldModel, ExpensiveModel
)
from .serializers import (
    RootModelSerializer, ParentSerializer, ItemSerializer,
    SlugRefChildSerializer, ExpensiveSerializer,
)


def cache_users_own_data(qs, req):
    if req.user.is_authenticated:
        return qs.filter(user=req.user)
    else:
        return qs.none()


rest.router.register_model(
    RootModel,
    url="",
    lookup="slug",
    serializer=RootModelSerializer,
    fields="__all__",
)
rest.router.register_model(
    UserManagedModel,
    fields="__all__",
    cache_filter=cache_users_own_data,
)
rest.router.register_model(
    Parent,
    serializer=ParentSerializer,
    fields="__all__",
)
rest.router.register_model(
    Child,
    per_page=100,
    url="children",
    fields="__all__",
)
rest.router.register_model(
    ItemType,
    fields="__all__",
    cache="all",
)
rest.router.register_model(
    Item,
    serializer=ItemSerializer,
    fields="__all__",
    cache="none",
)
rest.router.register_model(
    FileModel,
    fields="__all__",
)
rest.router.register_model(
    ImageModel,
    fields="__all__",
)
rest.router.register_model(
    SlugModel,
    lookup="code",
    fields="__all__",
)
rest.router.register_model(
    SlugRefParent,
    fields="__all__",
)
rest.router.register_model(
    SlugRefChild,
    url="slugrefchildren",
    serializer=SlugRefChildSerializer,
    fields="__all__",
)
rest.router.register_model(
    DateModel,
    fields="__all__",
)
rest.router.register_model(
    ChoiceModel,
    fields="__all__",
)
rest.router.register_model(
    TranslatedModel,
    fields="__all__",
)
rest.router.register_model(
    CharFieldModel,
    fields="__all__",
)
rest.router.register_model(
    ExpensiveModel,
    serializer=ExpensiveSerializer,
    queryset=ExpensiveModel.objects.defer('expensive', 'more_expensive'),
)

rest.router.add_page("rest_context", {})
rest.router.add_page("auth_context", {})
rest.router.add_page("script_context", {})



from django.db import models
import pystache


class LabelModel(models.Model):
    wq_label_template = "{{name}}"

    def __str__(self):
        return pystache.render(self.wq_label_template, self)

    class Meta:
        abstract = True



from rest_framework.pagination import PageNumberPagination
from collections import OrderedDict
from rest_framework.response import Response


class Pagination(PageNumberPagination):
    page_size_query_param = 'limit'

    def paginate_queryset(self, queryset, request, view=None):
        data = super(Pagination, self).paginate_queryset(
            queryset, request, view
        )
        if not view or not getattr(view, 'router', None):
            return data

        if request.accepted_renderer.format != 'json':
            return data

        non_format_kwargs = [
            kwarg for kwarg in
            list(view.kwargs.keys()) + list(request.GET.keys())
            if kwarg != 'format'
        ]
        if view.action != 'list' or any(non_format_kwargs):
            return data

        conf = view.router.get_model_config(queryset.model)
        cache = conf.get('cache', 'first_page')

        if cache == 'first_page':
            return data
        elif cache == 'all':
            return list(queryset)
        elif cache == 'none':
            return []
        elif cache == 'filter':
            cache_filter = view.router.get_cache_filter_for_model(
                queryset.model
            )
            return list(cache_filter(queryset, self.request))

        return data

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            # DRF default metadata
            ('count', self.page.paginator.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),

            # wq.db additional metadata
            ('page', self.page.number),
            ('pages', self.page.paginator.num_pages),
            ('per_page', self.page.paginator.per_page),
            ('multiple', self.page.paginator.num_pages > 1),

            # Actual data ('results' in DRF)
            ('list', data)
        ]))



from rest_framework import serializers
from django.core.exceptions import ImproperlyConfigured
try:
    from django.contrib.gis.db import models as model_fields
    from django.contrib.gis.geos import GEOSGeometry
except (OSError, ImproperlyConfigured):
    from django.db import models as model_fields
    GEOSGeometry = None

from django.utils import timezone
from django.core.exceptions import FieldDoesNotExist
from collections import OrderedDict

from django.conf import settings

from rest_framework.utils import model_meta
from html_json_forms.serializers import JSONFormSerializer
from .model_tools import get_object_id
from .exceptions import ImproperlyConfigured


class GeometryField(serializers.Field):
    def to_representation(self, value):
        if value is None:
            return None
        import json
        return json.loads(value.geojson)

    def to_internal_value(self, value):
        import json
        if isinstance(value, dict):
            value = json.dumps(value)

        if not GEOSGeometry:
            raise Exception("Missing GDAL")

        geom = GEOSGeometry(value)
        srid = getattr(settings, 'SRID', 4326)

        if 'crs' in value and value['crs'].get('type', None) == 'name':
            name = value['crs']['properties']['name']
            if name.startswith('urn:ogc:def:crs:EPSG::'):
                geom.srid = int(name.replace('urn:ogc:def:crs:EPSG::', ''))

        if geom.srid is None:
            geom.srid = 4326
        if geom.srid != srid:
            geom.transform(srid)
        return geom


class LookupRelatedField(serializers.SlugRelatedField):
    def __init__(self, router, model, **kwargs):
        self.router = router
        self.model = model
        super(serializers.SlugRelatedField, self).__init__(**kwargs)

    @property
    def slug_field(self):
        if not self.router:
            return 'pk'
        return self.router.get_lookup_for_model(self.model)


class LocalDateTimeField(serializers.ReadOnlyField):
    def to_representation(self, value):
        if value is None:
            return None
        if value.tzinfo:
            value = timezone.localtime(value)
        return value.strftime('%Y-%m-%d %I:%M %p')


MODEL_BOOLEAN_FIELDS = (
    model_fields.BooleanField,
    model_fields.NullBooleanField,
)


class BooleanLabelField(serializers.ReadOnlyField):
    def to_representation(self, value):
        if value is None:
            return None
        elif value:
            return 'Yes'
        else:
            return 'No'


class BlankCharField(serializers.CharField):
    def run_validation(self, data=serializers.empty):
        if self.allow_blank and data is None:
            return ''
        return super(BlankCharField, self).run_validation(data)


class BaseModelSerializer(JSONFormSerializer, serializers.ModelSerializer):
    xlsform_types = OrderedDict((
        (serializers.ImageField, 'image'),
        (serializers.FileField, 'binary'),
        (serializers.DateField, 'date'),
        (serializers.DateTimeField, 'dateTime'),
        (serializers.FloatField, 'decimal'),
        (GeometryField, 'geoshape'),
        (serializers.IntegerField, 'int'),
        (serializers.CharField, 'string'),
        (serializers.ChoiceField, 'select one'),
        (serializers.TimeField, 'time'),
    ))

    def get_fields_for_config(self):
        self._for_wq_config = True
        fields = self.get_fields()
        del self._for_wq_config
        for name, field in list(fields.items()):
            if name == 'id' or field.read_only:
                fields.pop(name)
            elif isinstance(field, serializers.NullBooleanField):
                fields[name] = serializers.ChoiceField(
                    choices=self.get_boolean_choices(field),
                    required=False,
                )
            elif isinstance(field, serializers.BooleanField):
                fields[name] = serializers.ChoiceField(
                    choices=self.get_boolean_choices(field),
                    required=field.required,
                )
        return fields

    def get_boolean_choices(self, field):
        return [(True, 'Yes'), (False, 'No')]

    def get_wq_config(self):
        fields = []
        nested_fields = []
        has_geo_fields = False

        overrides = getattr(self.Meta, 'wq_field_config', {})

        for name, field in self.get_fields_for_config().items():
            info = self.get_wq_field_info(name, field)

            if info['type'].startswith('geo') or info['name'] == 'latitude':
                has_geo_fields = True

            if info['name'] in overrides:
                info.update(overrides[info['name']])

            if info['type'] == 'repeat':
                nested_fields.append(info)
            else:
                fields.append(info)

        config = getattr(self.Meta, 'wq_config', {}).copy()
        config['form'] = fields + nested_fields
        if 'name' not in config:
            config['name'] = self.Meta.model._meta.model_name
        if has_geo_fields and 'map' not in config:
            config['map'] = True
        if 'label_template' not in config:
            label_template = getattr(
                self.Meta.model, 'wq_label_template', None
            )
            if label_template:
                config['label_template'] = label_template
        return config

    def get_wq_field_info(self, name, field, model=None):
        info = {
            'name': name,
            'label': field.label or name.replace('_', ' ').title(),
        }
        if field.required:
            info.setdefault('bind', {})
            info['bind']['required'] = True

        if field.help_text:
            info['hint'] = field.help_text

        if isinstance(field, serializers.ChoiceField):
            info['choices'] = [{
                'name': cname,
                'label': label,
            } for cname, label in field.choices.items()]
        elif getattr(field, 'max_length', None):
            if not isinstance(field, serializers.FileField):
                info['wq:length'] = field.max_length

        if isinstance(field, serializers.ListSerializer):
            # Nested model with a foreign key to this one
            field.child.context['router'] = self.router
            if hasattr(field.child, 'get_wq_config'):
                child_config = field.child.get_wq_config()
            else:
                child_config = {'form': []}
            info['type'] = 'repeat'
            info['children'] = child_config['form']
            if 'initial' in child_config:
                info['initial'] = child_config['initial']

        elif isinstance(field, serializers.RelatedField):
            # Foreign key to a parent model
            info['type'] = 'string'
            info['name'] = info['name'].replace('_id', '')
            info['label'] = info['label'].replace(' Id', '')
            if hasattr(field, 'queryset'):
                fk = self.get_wq_foreignkey_info(field.queryset.model)
                if fk:
                    info['wq:ForeignKey'] = fk

        if 'type' not in info:
            info['type'] = 'string'

            for field_type, xlsform_type in self.xlsform_types.items():
                if isinstance(field, field_type):
                    info['type'] = xlsform_type
                    break

            if info['type'] == 'geoshape':
                model = model or self.Meta.model
                source = model._meta.get_field(name)
                if info['type'] == 'geoshape':
                    geom_type = getattr(source, 'geom_type', None)
                    if geom_type == 'POINT':
                        info['type'] = 'geopoint'
                    elif geom_type == 'LINESTRING':
                        info['type'] = 'geotrace'

            if info['type'] == 'string':
                model = model or self.Meta.model
                try:
                    source = model._meta.get_field(name)
                except FieldDoesNotExist:
                    pass
                else:
                    if source.get_internal_type() == "TextField":
                        info['type'] = "text"

        return info

    def get_wq_foreignkey_info(self, model):
        if not self.router or not self.router.model_is_registered(model):
            return None
        else:
            serializer = self.router.get_serializer_for_model(
                model
            )
            model_conf = serializer().get_wq_config()
            return model_conf['name']

    class Meta:
        wq_config = {}
        wq_field_config = {}


class ModelSerializer(BaseModelSerializer):
    add_label_fields = True

    def __init__(self, *args, **kwargs):
        mapping = self.serializer_field_mapping

        for model_field, serializer_field in list(mapping.items()):
            if serializer_field == serializers.CharField:
                mapping[model_field] = BlankCharField

        if GEOSGeometry:
            for field in ('Geometry', 'GeometryCollection',
                          'Point', 'LineString', 'Polygon',
                          'MultiPoint', 'MultiLineString', 'MultiPolygon'):
                mapping[getattr(model_fields, field + 'Field')] = GeometryField
        super(ModelSerializer, self).__init__(*args, **kwargs)

    @property
    def router(self):
        if 'view' not in self.context and 'router' not in self.context:
            return None
        if 'view' in self.context:
            return self.context['view'].router
        else:
            return self.context['router']

    @property
    def is_detail(self):
        if getattr(self.Meta, 'depth', 0) > 0:
            return True
        view = self.context.get('view', None)
        return view and view.action != "list"

    @property
    def is_edit(self):
        view = self.context.get('view', None)
        return view and view.action == 'edit'

    @property
    def is_geojson(self):
        request = self.context.get('request', None)
        if not request:
            return
        if request.accepted_renderer.format == 'geojson':
            return True
        return False

    @property
    def is_html(self):
        request = self.context.get('request', None)
        if not request:
            return
        if request.accepted_renderer.format == 'html':
            return True
        return False

    @property
    def is_config(self):
        return getattr(self, '_for_wq_config', False)

    def get_fields(self, *args, **kwargs):
        fields = super(ModelSerializer, self).get_fields(*args, **kwargs)
        fields = self.update_id_fields(fields)
        fields.update(self.get_label_fields(fields))
        exclude = set()

        def get_exclude(meta_name):
            return set(getattr(self.Meta, meta_name, []))

        if not self.is_detail and not self.is_config:
            exclude |= get_exclude('list_exclude')
            if self.is_html:
                exclude |= get_exclude('html_list_exclude')

        if self.is_config:
            exclude |= get_exclude('config_exclude')

        for field in list(exclude):
            fields.pop(field, None)

        return fields

    def update_id_fields(self, fields):
        if 'id' not in self._declared_fields:
            lookup = getattr(self.Meta, 'wq_config', {}).get('lookup', None)
            if lookup and lookup != 'id':
                fields['id'] = serializers.ReadOnlyField(source=lookup)

        id_fields = {}
        info = model_meta.get_field_info(self.Meta.model)

        # In list views, remove [fieldname] as an attribute in favor of
        # [fieldname]_id.
        extra_kwargs = self.get_extra_kwargs()
        for name, field in info.forward_relations.items():
            include = getattr(self.Meta, "fields", [])
            if include == "__all__":
                include = []
            exclude = getattr(self.Meta, "exclude", [])
            if name in exclude or (include and name not in include):
                fields.pop(name, None)
                continue

            default_field = fields[name]
            auto_related_field = (serializers.Serializer, LookupRelatedField)
            if not isinstance(default_field, auto_related_field):
                continue

            if name + '_id' not in fields:
                id_field, id_field_kwargs = self.build_relational_field(
                    name, field
                )
                extra_field_kwargs = extra_kwargs.get(name, {})
                extra_field_kwargs.update(
                    extra_kwargs.get(name + '_id', {})
                )
                self.include_extra_kwargs(
                    id_field_kwargs,
                    extra_field_kwargs,
                )
                id_field_kwargs['source'] = name
                id_fields[name] = id_field(**id_field_kwargs)

            if name in fields:
                # Update/remove DRF default foreign key field (w/o _id)
                if self.is_detail and isinstance(
                        default_field, serializers.Serializer):
                    # Nested representation, keep for detail template context
                    fields[name].read_only = True
                else:
                    # Otherwise we don't need this field
                    del fields[name]

        if not id_fields:
            return fields

        # Ensure _id fields show up in the same order they appear on model
        updated_fields = OrderedDict()
        for field in self.Meta.model._meta.fields:
            name = field.name
            if name in fields:
                updated_fields[name] = fields.pop(name)
            if name in id_fields:
                updated_fields[name + '_id'] = id_fields.pop(name)

        if fields:
            updated_fields.update(fields)

        if id_fields:
            for name in id_fields:
                updated_fields[name + '_id'] = id_fields[name]

        return updated_fields

    def get_label_fields(self, default_fields):
        if not self.add_label_fields:
            return {}
        fields = {}

        exclude = getattr(self.Meta, 'exclude', [])
        if 'label' not in exclude and 'label' not in default_fields:
            fields['label'] = serializers.ReadOnlyField(source='__str__')

        info = model_meta.get_field_info(self.Meta.model)

        # Add labels for dates and fields with choices
        for name, field in info.fields.items():
            if name in getattr(self.Meta, "exclude", []):
                continue
            if name + '_label' in default_fields:
                continue
            if isinstance(field, model_fields.DateTimeField):
                fields[name + '_label'] = LocalDateTimeField(source=name)
            if field.choices:
                fields[name + '_label'] = serializers.ReadOnlyField(
                    source='get_%s_display' % name
                )
            if isinstance(field, MODEL_BOOLEAN_FIELDS):
                fields[name + '_label'] = BooleanLabelField(
                    source=name,
                )

        # Add labels for related fields
        for name, field in info.forward_relations.items():
            if name in getattr(self.Meta, "exclude", []):
                continue
            if name + '_label' in default_fields:
                continue

            f, field_kwargs = self.build_relational_field(
                name, info.forward_relations[name],
            )
            label_field_kwargs = {
                'source': name,
            }
            if field_kwargs.get('many', None):
                label_field_kwargs['many'] = field_kwargs['many']
            fields[name + '_label'] = serializers.StringRelatedField(
                **label_field_kwargs
            )

        return fields

    @classmethod
    def for_model(cls, model_class, include_fields=None):
        class Serializer(cls):
            class Meta(getattr(cls, 'Meta', object)):
                model = model_class
                if include_fields:
                    fields = include_fields
        return Serializer

    @classmethod
    def for_depth(cls, serializer_depth):
        class Serializer(cls):
            class Meta(getattr(cls, 'Meta', object)):
                depth = serializer_depth
        return Serializer

    def build_relational_field(self, field_name, relation_info):
        if isinstance(relation_info.related_model, str):
            raise ImproperlyConfigured(
                "%s could not be resolved to a model class"
                % relation_info.related_model,
            )

        field_class, field_kwargs = super(
            ModelSerializer, self
        ).build_relational_field(field_name, relation_info)
        if field_class == self.serializer_related_field:
            field_class = LookupRelatedField
            field_kwargs['router'] = self.router
            field_kwargs['model'] = relation_info.related_model
        return field_class, field_kwargs

    def build_nested_field(self, field_name, relation_info, nested_depth):
        field_class, field_kwargs = super(
            ModelSerializer, self
        ).build_nested_field(
            field_name, relation_info, nested_depth
        )
        field_kwargs['required'] = False
        if self.router:
            field_class = self.router.get_serializer_for_model(
                relation_info.related_model, nested_depth
            )
        return field_class, field_kwargs

    def add_lookups(self, context):
        # Mimic _addLookups in wq.app/app.js
        if not self.router:
            return

        conf = self.get_wq_config()
        for field in conf['form']:
            if 'choices' in field:
                # CharField choices
                context[field['name'] + '_choices'] = [{
                    'name': choice['name'],
                    'label': choice['label'],
                    'selected': (
                        choice['name'] == context.get(field['name'], '')
                    ),
                } for choice in field['choices']]

            elif 'wq:ForeignKey' in field:
                choices = self.get_lookup_choices(field, context)
                if choices:
                    context[field['name'] + '_list'] = choices

            elif 'children' in field:
                rows = context.get(field['name'])
                if isinstance(rows, list) and len(rows) > 0:
                    serializer = self.get_fields().get(field['name'])
                    serializer.child.context.update(
                        router=self.router,
                        request=self.context.get('request')
                    )
                    if serializer and hasattr(serializer.child, 'add_lookups'):
                        for row in rows:
                            serializer.child.add_lookups(row)

    def get_lookup_choices(self, field, context):
        model_name = field['wq:ForeignKey']
        model_conf = self.router.get_model_config(field['wq:ForeignKey'])
        if not model_conf:
            return

        qs = self.router.get_queryset_for_model(
            model_name, self.context.get('request')
        )
        if field.get('filter', None):
            qs = qs.filter(**self.compute_filter(
                field['filter'],
                model_conf,
                context
            ))
        choices = self.serialize_choices(qs, field)
        self.set_selected(choices, context.get(field['name'] + '_id', ''))
        return choices

    def compute_filter(self, filter, model_conf, context):
        def render(value):
            import pystache
            result = pystache.render(value, context)
            if result == '':
                return None
            elif result.isdigit():
                result = int(result)
            return result

        fk_lookups = {}
        for field in model_conf['form']:
            if 'wq:ForeignKey' not in field:
                continue
            lookup = self.router.get_lookup_for_model(
                field['wq:ForeignKey']
            )
            if lookup and lookup != 'pk':
                fk_lookups['%s_id' % field['name']] = '%s__%s' % (
                    field['name'], lookup
                )

        computed_filter = {}
        for key, values in filter.items():
            if not isinstance(values, list):
                values = [values]
            values = [
                render(value) if '{{' in value else value
                for value in values
            ]

            if key in fk_lookups:
                key = fk_lookups[key]

            if len(values) > 1:
                computed_filter[key + '__in'] = values
            else:
                computed_filter[key] = values[0]

        return computed_filter

    def serialize_choices(self, qs, field):
        return [{
            'id': get_object_id(obj),
            'label': str(obj)
        } for obj in qs]

    def set_selected(self, choices, value):
        for choice in choices:
            if choice['id'] == value:
                choice['selected'] = True



from wq.db import rest
from .models import (
    NaturalKeyParent,
    ModelWithNaturalKey
)
from wq.db.patterns.serializers import NaturalKeyModelSerializer


rest.router.register_model(
    NaturalKeyParent,
    fields="__all__",
)
rest.router.register_model(
    ModelWithNaturalKey,
    serializer=NaturalKeyModelSerializer,
    fields="__all__",
)



from django.db import models, connections
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import (
    GenericForeignKey, GenericRelation
)
from natural_keys.models import NaturalKeyModelManager, NaturalKeyModel
from ..base.models import LabelModel
from django.template.defaultfilters import slugify
from django.conf import settings
INSTALLED = ('wq.db.patterns.identify' in settings.INSTALLED_APPS)


WQ_IDENTIFIER_ORDER = getattr(
    settings, "WQ_IDENTIFIER_ORDER",
    ["-is_primary", "authority", "name"]
)


def find_unique_slug(name, queryset):
    if not name:
        maxobj = queryset.order_by('-pk').first()
        name = str(maxobj.pk + 1 if maxobj else 1)
    if len(name) > 45:
        name = name[:45]
    slug = slugify(name)
    exists = queryset.filter(slug=slug)
    num = ''
    while exists.count() > 0:
        slug = slugify('%s %s' % (name, num))
        exists = queryset.filter(slug=slug)
        if num == '':
            num = 1
        else:
            num += 1
    return slug


class IdentifierManager(models.Manager):
    def filter_by_identifier(self, identifier):
        return self.filter(
            models.Q(slug=identifier) | models.Q(name=identifier)
        ).order_by('-is_primary')

    def resolve(self, identifiers, exclude_apps=[]):
        resolved = None
        unresolved = None
        for identifier in identifiers:
            ids = self.filter_by_identifier(identifier)
            if exclude_apps:
                ids = ids.exclude(content_type__app_label__in=exclude_apps)
            if connections[self.db].features.can_distinct_on_fields:
                items = ids.order_by(
                    'content_type', 'object_id'
                ).distinct('content_type', 'object_id').count()
            else:
                distinct_ids = set()
                for ident in ids:
                    distinct_ids.add((ident.content_type_id, ident.object_id))
                items = len(distinct_ids)
            if items == 1:
                if resolved is None:
                    resolved = {}
                resolved[identifier] = ids[0]
            else:
                if unresolved is None:
                    unresolved = {}
                unresolved[identifier] = ids
        return resolved, unresolved

    # Default implementation of get_or_create doesn't work well with generics
    def get_or_create(self, defaults={}, **kwargs):
        try:
            return self.get(**kwargs), False
        except self.model.DoesNotExist:
            kwargs.update(defaults)
            return self.create(**kwargs), True


class Identifier(models.Model):
    name = models.CharField(max_length=255, db_index=True)
    slug = models.SlugField(blank=True)
    authority = models.ForeignKey(
        'Authority', models.CASCADE, blank=True, null=True,
    )
    is_primary = models.BooleanField(default=False)

    # Identifer can contain a pointer to any model
    content_type = models.ForeignKey(ContentType, models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey()

    objects = IdentifierManager()

    @property
    def url(self):
        if (not self.authority or not self.authority.object_url):
            return None
        else:
            return self.authority.object_url % self.slug

    def save(self, *args, **kwargs):
        if not self.slug:
            model = self.content_type.model_class()
            queryset = model.objects.exclude(pk=self.object_id)
            self.slug = find_unique_slug(self.name, queryset)

        if self.is_primary and not self.pk:
            exist = self.content_object.primary_identifier
            if exist:
                # Primary identifier already exists for this object, update
                # instead of adding another
                self.pk = exist.pk
                kwargs.pop('force_insert', None)
                kwargs['force_update'] = True

        super(Identifier, self).save(*args, **kwargs)

        if self.is_primary:
            obj = self.content_object
            if self.name != obj.name or self.slug != obj.slug:
                obj.name = self.name
                obj.slug = self.slug
                obj.save()

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'wq_identifier'
        ordering = WQ_IDENTIFIER_ORDER
        abstract = not INSTALLED


class IdentifiedModelManager(NaturalKeyModelManager):
    def get_by_identifier(self, identifier, auto_create=False):
        searches = [
            {'slug': identifier},
            {'name': identifier},
            {'identifiers__slug': identifier},
            {'identifiers__name': identifier},
        ]

        obj = None
        for search in searches:
            try:
                obj = self.get(**search)
                break
            except (ValueError, self.model.DoesNotExist):
                pass

        if obj is None:
            if auto_create:
                obj = self.create_by_natural_key(identifier)
            else:
                name = self.model._meta.object_name
                raise self.model.DoesNotExist(
                    '%s "%s" does not exist' % (name, identifier)
                )

        return obj

    def get_by_natural_key(self, identifier):
        return self.get_by_identifier(identifier)

    def create_by_natural_key(self, identifier, **kwargs):
        defaults = kwargs.get('defaults') or {}
        return self.create(name=identifier, **defaults)


class IdentifiedModel(NaturalKeyModel, LabelModel):
    name = models.CharField(
        max_length=255, blank=True, db_index=True
    )
    slug = models.CharField(max_length=255, blank=True, unique=True)

    identifiers = GenericRelation(Identifier)
    objects = IdentifiedModelManager()

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = find_unique_slug(self.name, type(self).objects)
        if not self.name:
            self.name = self.slug
        super(IdentifiedModel, self).save()
        ident, is_new = self.identifiers.get_or_create(
            is_primary=True,
            defaults={
                'name': self.name,
                'slug': self.slug,
            }
        )
        if not is_new and (ident.name != self.name or ident.slug != self.slug):
            ident.name = self.name
            ident.slug = self.slug
            ident.save()

    @property
    def primary_identifier(self):
        return self.identifiers.filter(is_primary=True).first()

    class Meta:
        ordering = ['name']
        abstract = True


class Authority(LabelModel):
    name = models.CharField(max_length=255)
    homepage = models.URLField(null=True, blank=True)
    object_url = models.URLField(null=True, blank=True)

    class Meta:
        verbose_name_plural = 'authorities'
        db_table = 'wq_identifiertype'
        abstract = not INSTALLED



from django.db import models
from wq.db.patterns import models as patterns


class IdentifiedModel(patterns.IdentifiedModel):
    pass


class FilterableModel(patterns.LabelModel):
    name = models.CharField(max_length=10)
    parent = models.ForeignKey(
        IdentifiedModel, models.CASCADE
    )


class CustomPatternModel(models.Model):
    name = models.CharField(max_length=10)


class CustomAttachment(models.Model):
    name = models.CharField(max_length=10)
    parent = models.ForeignKey(
        CustomPatternModel, models.CASCADE, related_name='attachments',
    )


class CustomTypedPatternModel(models.Model):
    name = models.CharField(max_length=10)


class CustomType(models.Model):
    name = models.CharField(max_length=10)

    def __str__(self):
        return self.name


class CustomTypedAttachment(models.Model):
    name = models.CharField(max_length=10, null=True, blank=True)
    value = models.FloatField(null=True, blank=True)
    type = models.ForeignKey(CustomType, models.CASCADE)
    parent = models.ForeignKey(
        CustomTypedPatternModel,
        models.CASCADE,
        related_name='attachments'
    )


class Campaign(models.Model):
    pass


class Attribute(models.Model):
    name = models.CharField(max_length=10)
    campaign = models.ForeignKey(Campaign, models.CASCADE)
    is_active = models.BooleanField()
    category = models.CharField(max_length=10, blank=True)


class Entity(models.Model):
    campaign = models.ForeignKey(Campaign, models.CASCADE)

    class Meta:
        verbose_name_plural = 'entities'


class Value(models.Model):
    attribute = models.ForeignKey(Attribute, models.CASCADE)
    entity = models.ForeignKey(Entity, models.CASCADE, related_name='values')
